@@ -169,6 +169,7 @@ static attr_handler2 attr_handlers[] = {
     ADD_ATTR_HANDLER(CKA_TPM2_OBJAUTH_ENC, TYPE_BYTE_HEX_STR),
     ADD_ATTR_HANDLER(CKA_TPM2_PUB_BLOB, TYPE_BYTE_HEX_STR),
     ADD_ATTR_HANDLER(CKA_TPM2_PRIV_BLOB, TYPE_BYTE_HEX_STR),
+    ADD_ATTR_HANDLER(CKA_TPM2_ENC_BLOB, TYPE_BYTE_HEX_STR),
 };
 
 static attr_handler2 default_handler = { .memtype = 0 };
@@ -1212,54 +1213,32 @@ error:
     return rv;
 }
 
-CK_RV attr_list_update_entry(attr_list *attrs, CK_ATTRIBUTE_PTR untrusted_attr) {
-    assert(attrs);
-    assert(untrusted_attr);
+static CK_RV check_attr(CK_ATTRIBUTE_PTR a, CK_BYTE memtype) {
 
-    CK_ATTRIBUTE_TYPE t = untrusted_attr->type;
-
-    attr_handler2 *handler = attr_lookup(t);
-
-    CK_ATTRIBUTE_PTR found = attr_get_attribute_by_type(attrs, t);
-    if (!found) {
-        LOGE("Attribute entry not found");
-        return CKR_GENERAL_ERROR;
-    }
-
-    /* the existing type memory should match the expected type memory */
-    CK_BYTE expected_memory_type = type_from_ptr(found->pValue,
-            found->ulValueLen);
-
-    /* internal state check */
-    if (expected_memory_type != handler->memtype) {
-        LOGE("expected memory(%u-%s) != handler memory(%u-%s)",
-            expected_memory_type, type_to_str(expected_memory_type),
-            handler->memtype, type_to_str(handler->memtype));
-        return CKR_GENERAL_ERROR;
-    }
+    CK_BBOOL bbool;
+    CK_RV rv = CKR_GENERAL_ERROR;
 
     /* validate sizes */
-    void *pValue = untrusted_attr->pValue;
-    CK_ULONG ulValueLen = untrusted_attr->ulValueLen;
+    CK_ULONG ulValueLen = a->ulValueLen;
 
-    switch (handler->memtype) {
+    switch (memtype) {
     case TYPE_BYTE_INT:
         if (ulValueLen != sizeof(CK_ULONG)) {
             LOGE("ulValueLen(%lu) != sizeof(CK_ULONG)", ulValueLen);
-            return CKR_ATTRIBUTE_VALUE_INVALID;
+            return CKR_MECHANISM_PARAM_INVALID;
         }
         break;
     case TYPE_BYTE_BOOL:
-        if (ulValueLen != sizeof(CK_BBOOL)) {
-            LOGE("ulValueLen(%lu) != sizeof(CK_BBOOL)", ulValueLen);
-            return CKR_ATTRIBUTE_VALUE_INVALID;
+        rv = attr_CK_BBOOL(a, &bbool);
+        if (rv != CKR_OK) {
+            return rv;
         }
         break;
     case TYPE_BYTE_INT_SEQ:
         if (ulValueLen % sizeof(CK_ULONG)) {
             LOGE("ulValueLen(%lu) %% sizeof(CK_ULONG)",
                     ulValueLen % sizeof(CK_ULONG));
-            return CKR_ATTRIBUTE_VALUE_INVALID;
+            return CKR_MECHANISM_PARAM_INVALID;
         }
         break;
     case TYPE_BYTE_HEX_STR:
@@ -1267,10 +1246,31 @@ CK_RV attr_list_update_entry(attr_list *attrs, CK_ATTRIBUTE_PTR untrusted_attr)
         break;
     default:
         LOGE("Unknown data type representation, got: %u",
-                handler->memtype);
+                memtype);
         return CKR_GENERAL_ERROR;
     }
 
+    return CKR_OK;
+}
+
+CK_RV attr_list_update_entry(attr_list *attrs, CK_ATTRIBUTE_PTR untrusted_attr) {
+    assert(attrs);
+    assert(untrusted_attr);
+
+    CK_ATTRIBUTE_TYPE t = untrusted_attr->type;
+
+    attr_handler2 *handler = attr_lookup(t);
+    assert(handler);
+
+    CK_ATTRIBUTE_PTR found = attr_get_attribute_by_type(attrs, t);
+    if (!found) {
+        LOGE("Attribute entry not found");
+        return CKR_GENERAL_ERROR;
+    }
+
+    void *pValue = untrusted_attr->pValue;
+    CK_ULONG ulValueLen = untrusted_attr->ulValueLen;
+
     if (ulValueLen != found->ulValueLen) {
         void *new_pValue = type_zrealloc(found->pValue, ulValueLen, handler->memtype);
         if (!new_pValue) {
@@ -1293,6 +1293,16 @@ CK_RV attr_list_append_entry(attr_list **attrs, CK_ATTRIBUTE_PTR untrusted_attr)
     assert(*attrs);
     assert(untrusted_attr);
 
+    CK_ATTRIBUTE_TYPE t = untrusted_attr->type;
+
+    attr_handler2 *handler = attr_lookup(t);
+    assert(handler);
+
+    CK_RV rv = check_attr(untrusted_attr, handler->memtype);
+    if (rv != CKR_OK) {
+        return rv;
+    }
+
     attr_list *new_item = NULL;
     bool res = attr_typify(untrusted_attr, 1, &new_item);
     if (!res) {
@@ -1344,4 +1354,5 @@ UTILS_GENERIC_ATTR_TYPE_CONVERT(CK_KEY_TYPE);
 
 UTILS_GENERIC_EXTRACTOR(CKA_PRIVATE, CK_BBOOL);
 UTILS_GENERIC_EXTRACTOR(CKA_CLASS,   CK_OBJECT_CLASS);
+UTILS_GENERIC_EXTRACTOR(CKA_KEY_TYPE,   CK_KEY_TYPE);
 UTILS_GENERIC_EXTRACTOR(CKA_TOKEN, CK_BBOOL);
