@@ -119,6 +119,13 @@ static CK_RV rsa_pss_synthesizer(mdetail *m, CK_MECHANISM_PTR mech, attr_list *a
 static CK_RV rsa_pkcs_hash_synthesizer(mdetail *m, CK_MECHANISM_PTR mech, attr_list *attrs,
         CK_BYTE_PTR inbuf, CK_ULONG inlen,
         CK_BYTE_PTR outbuf, CK_ULONG_PTR outlen);
+static CK_RV aes_cbc_synthesizer(mdetail *m, CK_MECHANISM_PTR mech, attr_list *attrs,
+        CK_BYTE_PTR inbuf, CK_ULONG inlen,
+        CK_BYTE_PTR outbuf, CK_ULONG_PTR outlen);
+static CK_RV aes_cbc_unsynthesizer(mdetail *mdtl,
+        CK_MECHANISM_PTR mech, attr_list *attrs,
+        CK_BYTE_PTR inbuf, CK_ULONG inlen,
+        CK_BYTE_PTR outbuf, CK_ULONG_PTR outlen);
 static CK_RV rsa_pss_get_halg(CK_MECHANISM_PTR mech, CK_MECHANISM_TYPE_PTR halg);
 static CK_RV rsa_oaep_get_halg(CK_MECHANISM_PTR mech, CK_MECHANISM_TYPE_PTR halg);
 static CK_RV sha1_get_halg(CK_MECHANISM_PTR mech, CK_MECHANISM_TYPE_PTR halg);
@@ -167,8 +174,10 @@ static const mdetail_entry _g_mechs_templ[] = {
     { .type = CKM_AES_KEY_GEN, .flags = mf_is_keygen|mf_aes },
 
     { .type = CKM_AES_CBC,    .flags = mf_encrypt|mf_decrypt|mf_aes, .get_tpm_opdata = tpm_aes_cbc_get_opdata },
+    { .type = CKM_AES_CBC_PAD, .flags = mf_encrypt|mf_decrypt|mf_aes|mf_force_synthetic, .get_tpm_opdata = tpm_aes_cbc_get_opdata, .synthesizer = aes_cbc_synthesizer, .unsynthesizer = aes_cbc_unsynthesizer },
     { .type = CKM_AES_CFB128, .flags = mf_encrypt|mf_decrypt|mf_aes, .get_tpm_opdata = tpm_aes_cfb_get_opdata },
     { .type = CKM_AES_ECB,    .flags = mf_encrypt|mf_decrypt|mf_aes, .get_tpm_opdata = tpm_aes_ecb_get_opdata },
+    { .type = CKM_AES_CTR,    .flags = mf_encrypt|mf_decrypt|mf_aes, .get_tpm_opdata = tpm_aes_ctr_get_opdata },
 
     /* hashing */
     { .type = CKM_SHA_1,  .flags = mf_is_digester|mf_aes, .validator = hash_validator, .get_digester = sha1_get_digester },
@@ -220,7 +229,7 @@ static CK_RV mech_init(tpm_ctx *tctx, mdetail *m) {
     assert(tpm_mechs_len <= m->mdetail_len);
 
     /*
-     * Update whether or not the TPM supports it ot not
+     * Update whether or not the TPM supports it or not
      * and any other metadata
      */
     CK_ULONG i;
@@ -1034,6 +1043,30 @@ CK_RV rsa_pkcs_hash_synthesizer(mdetail *mdtl,
     return rsa_pkcs_synthesizer(mdtl, mech, attrs, hdr_buf, total_size, outbuf, outlen);
 }
 
+static CK_RV aes_cbc_synthesizer(mdetail *mdtl,
+        CK_MECHANISM_PTR mech, attr_list *attrs,
+        CK_BYTE_PTR inbuf, CK_ULONG inlen,
+        CK_BYTE_PTR outbuf, CK_ULONG_PTR outlen) {
+
+    UNUSED(mdtl);
+    UNUSED(mech);
+    UNUSED(attrs);
+
+    return apply_pkcs7_pad(inbuf, inlen, outbuf, outlen);
+}
+
+static CK_RV aes_cbc_unsynthesizer(mdetail *mdtl,
+        CK_MECHANISM_PTR mech, attr_list *attrs,
+        CK_BYTE_PTR inbuf, CK_ULONG inlen,
+        CK_BYTE_PTR outbuf, CK_ULONG_PTR outlen) {
+
+    UNUSED(mdtl);
+    UNUSED(mech);
+    UNUSED(attrs);
+
+    return remove_pkcs7_pad(inbuf, inlen, outbuf, outlen);
+}
+
 static CK_RV get_rsa_mechinfo(tpm_ctx *tctx, CK_MECHANISM_INFO_PTR info) {
 
     CK_ULONG min = 0;
