@@ -61,7 +61,7 @@ WEAK CK_RV token_min_init(token *t) {
     }
 
     /*
-     * Initalize the per-token mechanism details table
+     * Initialize the per-token mechanism details table
      */
     rv = mdetail_new(t->tctx, &t->mdtl, t->config.pss_sigs_good);
     if (rv != CKR_OK) {
@@ -89,12 +89,21 @@ void token_reset(token *t) {
      */
 }
 
-void token_free_list(token *t, size_t len) {
+void token_free_list(token **tok_ptr, size_t *ptr_len) {
+
+    size_t len = *ptr_len;
+    token *t = *tok_ptr;
+    *tok_ptr = NULL;
+    *ptr_len = 0;
+    if (!t) {
+        return;
+    }
 
     size_t i;
     for (i=0; i < len; i++) {
         token_free(&t[i]);
     }
+    memset(t, 0, sizeof(*t) * len);
     free(t);
 }
 
@@ -391,7 +400,7 @@ CK_RV token_init(token *t, CK_BYTE_PTR pin, CK_ULONG pin_len, CK_BYTE_PTR label)
 
     rv = slot_add_uninit_token();
     if (rv != CKR_OK) {
-        LOGW("Could not add unitialized token");
+        LOGW("Could not add uninitialized token");
     }
 
     rv =  CKR_OK;
