@@ -1,12 +1,22 @@
 # SPDX-License-Identifier: BSD-2-Clause
 import fcntl
+import io
 import os
 import sys
 import sqlite3
 import textwrap
 import yaml
 
-VERSION = 4
+from .pkcs11t import (
+    CKA_ALLOWED_MECHANISMS,
+    CKA_CLASS,
+    CKO_SECRET_KEY,
+    CKA_KEY_TYPE,
+    CKM_AES_CBC_PAD,
+    CKK_AES
+)
+
+VERSION = 5
 
 #
 # With Db() as db:
@@ -63,7 +73,7 @@ class Db(object):
         return x
 
     def rmtoken(self, label):
-        # This works on the premise of a cascading delete tied by foriegn
+        # This works on the premise of a cascading delete tied by foreign
         # key relationships.
         self._conn.execute('DELETE from tokens WHERE label=?', (label, ))
 
@@ -74,7 +84,7 @@ class Db(object):
         return x
 
     def rmprimary(self, pid):
-        # This works on the premise of a cascading delete tied by foriegn
+        # This works on the premise of a cascading delete tied by foreign
         # key relationships.
         self._conn.execute('DELETE from pobjects WHERE id=?', (pid, ))
 
@@ -194,15 +204,19 @@ class Db(object):
         c.execute(sql, list(tobject.values()))
         return c.lastrowid
 
-    def updatetertiary(self, tid, attrs):
-
-        c = self._conn.cursor()
+    @staticmethod
+    def _updatetertiary(db, tid, attrs):
+        c = db.cursor()
         attrs = yaml.safe_dump(attrs, canonical=True)
         values = [attrs, tid]
 
         sql = 'UPDATE tobjects SET attrs=? WHERE id=?'
         c.execute(sql, values)
 
+    def updatetertiary(self, tid, attrs):
+
+        self._updatetertiary(self._conn, tid, attrs)
+
     def updatepin(self, is_so, token, sealauth, sealpriv, sealpub=None):
 
         tokid = token['id']
@@ -382,6 +396,47 @@ class Db(object):
         s = 'ALTER TABLE pobjects2 RENAME TO pobjects;'
         dbbakcon.execute(s)
 
+    def _update_on_5(self, dbbakcon):
+        '''
+        Between version 4 and 5 of the DB the following changes need to be made:
+
+        Table tobjects:
+
+        The YAML attributes need to include CKM_AES_CBC_PAD and CKM_AES_CTR in the
+        CKM_ALLOWED_MECHANISMS list.
+        '''
+
+        c = dbbakcon.cursor()
+
+        c.execute('SELECT * from tobjects')
+        tobjs = c.fetchall()
+
+        for t in tobjs:
+            attrs = yaml.safe_load(io.StringIO(t['attrs']))
+
+            # IF the object is definitely a SECRET KEY of AES and has
+            # CKM_AES_CBC_PAD AND CKM_AES_CTR in allowed mechanisms, skip it.
+            if CKA_CLASS not in attrs or \
+                attrs[CKA_CLASS] != CKO_SECRET_KEY or \
+                CKA_KEY_TYPE not in attrs or \
+                attrs[CKA_KEY_TYPE] != CKK_AES or \
+                CKA_ALLOWED_MECHANISMS not in attrs \
+                (CKM_AES_CBC_PAD in attrs[CKA_ALLOWED_MECHANISMS] \
+                 and \
+                 CKM_AES_CTR in attrs[CKA_ALLOWED_MECHANISMS]):
+                continue
+
+            # Is an AES KEY and needs CKM_AES_CBC_PAD
+            if not CKM_AES_CBC_PAD in attrs[CKA_ALLOWED_MECHANISMS]:
+                attrs[CKA_ALLOWED_MECHANISMS].append(CKM_AES_CBC_PAD)
+
+            # Is an AES KEY and needs CKM_AES_CBC_PAD
+            if not CKM_AES_CTR in attrs[CKA_ALLOWED_MECHANISMS]:
+                attrs[CKA_ALLOWED_MECHANISMS].append(CKM_AES_CBC_PAD)
+
+            Db._updatetertiary(dbbakcon, t['id'], attrs)
+
+
     def update_db(self, old_version, new_version=VERSION):
 
         # were doing the update, so make a backup to manipulate
